'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ProgressBar = exports.activeRequests = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.setOriginalFetch = setOriginalFetch;
exports.progressBarFetch = progressBarFetch;
exports.setActiveRequests = setActiveRequests;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// Keeps track of how many requests are currently active.
var activeRequests = exports.activeRequests = 0;

/*
  The modes form a state machine with the following flow

  inactive -> active -> complete -
  ^                               |
  |                               |
  --------------------------------

  inactive: no animation is running the bar is invisible
  active: the animation is running slowly to 80%
  complete: the animation runs quickly to 100%
*/

var ProgressBar = exports.ProgressBar = function (_Component) {
  _inherits(ProgressBar, _Component);

  function ProgressBar() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, ProgressBar);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = ProgressBar.__proto__ || Object.getPrototypeOf(ProgressBar)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      mode: 'inactive'
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(ProgressBar, [{
    key: 'componentDidMount',


    // Start the initial tick
    value: function componentDidMount() {
      this.tick();
    }

    // Only render if the mode changes

  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps, nextState) {
      if (nextState.mode !== this.state.mode) {
        return true;
      }

      return false;
    }

    /**
     * The ProgressBar continuously checks the status of how many
     * requests are currently active, and will accordingly move
     * to another state.
     * 
     * It is important that the tick always calls itself after
     * a while or the cycle will be broken.
     * 
     * @memberof ProgressBar
     */

  }, {
    key: 'tick',
    value: function tick() {
      var _this2 = this;

      var mode = this.state.mode;

      if (mode === 'complete') {
        //console.log('complete: moving to inactive after 1 second to allow the close animation to complete');
        setTimeout(function () {
          _this2.moveToMode('inactive');
        }, 1000);
      } else if (mode === 'active') {
        if (activeRequests === 0) {
          //console.log('active: there are no more pending request move to complete if there are still no pending requests after 200 milliseconds');
          setTimeout(function () {
            if (activeRequests === 0) {
              //console.log('active: even after 200 milliseconds there are no more pending request, moving to complete');
              _this2.moveToMode('complete');
            } else {
              //console.log('active: after 200 milliseconds another request was pending instead of going to complete will stay active.');
              _this2.tick();
            }
          }, 200);
        } else {
          //console.log('active: there are still pending requests staying active');
          this.tickWithDelay();
        }
      } else {
        // mode === 'inactive'
        if (activeRequests > 0) {
          //console.log('inactive: there are pending request move to active if there are still pending requests after 100 milliseconds');
          setTimeout(function () {
            if (activeRequests > 0) {
              //console.log('inactive: even after 100 milliseconds there are pending request, moving to active to trigger animation');
              _this2.moveToMode('active');
            } else {
              //console.log('inactive: after 100 milliseconds there were no pending request, the requests was so fast that showing an animation is unnecessary, staying inactive');
              _this2.tick();
            }
          }, 100);
        } else {
          //console.log('inactive: no pending requests staying inactive');
          this.tickWithDelay();
        }
      }
    }
  }, {
    key: 'moveToMode',
    value: function moveToMode(mode) {
      var _this3 = this;

      this.setState({ mode: mode }, function () {
        _this3.tick();
      });
    }
  }, {
    key: 'tickWithDelay',
    value: function tickWithDelay() {
      var _this4 = this;

      setTimeout(function () {
        _this4.tick();
      }, 50);
    }
  }, {
    key: 'render',
    value: function render() {
      var mode = this.state.mode;

      var width = mode === 'complete' ? 100 : mode === 'inactive' ? 0 : 80;
      var animationSpeed = mode === 'complete' ? 0.8 : 30;
      var transition = mode === 'inactive' ? '' : 'width ' + animationSpeed + 's ease-in';

      var style = _extends({
        position: 'absolute',
        top: '0',
        zIndex: '9000',
        backgroundColor: '#f0ad4e',
        height: '4px',
        transition: transition,
        width: width + '%'
      }, this.props.style);

      return _react2.default.createElement('div', { className: 'react-fetch-progress-bar', style: style });
    }
  }]);

  return ProgressBar;
}(_react.Component);

// We store the fetch here as provided by the user.
var originalFetch = void 0;

function setOriginalFetch(nextOriginalFetch) {
  originalFetch = nextOriginalFetch;
}

/**
 * Wrapper around fetch: https://developer.mozilla.org/en/docs/Web/API/Fetch_API
 * 
 * It is used to monitor the number of requests which are currently
 * active. Each time a requests is made it increases the number of
 * requests, each time a request is finished, the number is decreased.
 * 
 * @export
 * @param {string} url The url you want to send a request to.
 * @param {RequestOptions} [options] The options you want to pass for that request
 * @returns {Promise<Response>} A Promise which returns a Response 
 */
function progressBarFetch(url, options) {
  exports.activeRequests = activeRequests += 1;

  return originalFetch(url, options).then(function (response) {
    exports.activeRequests = activeRequests -= 1;
    return response;
  }).catch(function (error) {
    exports.activeRequests = activeRequests -= 1;
    return Promise.reject(error);
  });
}

/**
 * Sets the number of activeRequests manually.
 * 
 * This method exists for testing purposes, so you should not
 * use it.
 * 
 * @export
 * @param {number} nextActiveRequest 
 */
function setActiveRequests(nextActiveRequest) {
  exports.activeRequests = activeRequests = nextActiveRequest;
}